@@ -22,6 +22,7 @@ from sqlalchemy.ext.declarative import declarative_base as dbase
 from sqlalchemy.ext.automap import automap_base
 from pathlib import Path
 import upcean
+import nanoid
 
 class RoutineDuration(BASE,Template):
     __tablename__="RoutineDuration"
@@ -39,17 +40,23 @@ RoutineDuration.metadata.create_all(ENGINE)
 class Exercise(BASE,Template):
     __tablename__="Exercise"
     exid=Column(Integer,primary_key=True)
-    Name=Column(String)
-    Note=Column(String)
-    Reps=Column(Integer) # 8 reps of 30 repcounts NAME
-    RepCount=Column(Integer) #30 times rep
-    CurrentRep=Column(Integer)
+    Name=Column(String,default=f'Generic Exercise Name [{nanoid.generate()}]')
+    Note=Column(String,default='')
+    Reps=Column(Integer,default=1) # 8 reps of 30 repcounts NAME
+    RepCount=Column(Integer,default=30) #30 times rep
+    CurrentRep=Column(Integer,default=0)
     cdt=Column(DateTime) #CurrentDateTime
     ldt=Column(DateTime) #LastDateTime
 
     def __init__(self,**kwargs):
         kwargs['__tablename__']=self.__tablename__
         self.init(**kwargs,)
+        if 'cdt' not in kwargs.keys():
+            self.cdt=datetime.now()
+
+        if 'ldt' not in kwargs.keys():
+            self.ldt=datetime.now()
+
 
 class Routine(BASE,Template):
     __tablename__="Routine"
@@ -97,6 +104,19 @@ e_data={'Name':{
             },
       }    
 
+def gethpv(routine):
+    with Session(ENGINE) as session:
+        if isinstance(routine,dict):
+            hpv=session.query(Routine).filter(Routine.Name==routine.get("Name")).order_by(Routine.precedence.desc()).first()
+        elif isinstance(routine,Routine):
+            hpv=session.query(Routine).filter(Routine.Name==getattr(routine,"Name")).order_by(Routine.precedence.desc()).first()
+        else:
+            raise Exception(f"Unsupported Type! {routine} -> type({type(routine)})")
+        #print(hpv)
+        if hpv:
+            return hpv.precedence
+        else:
+            return 0
 
 class ExerciseTracker:
     def newExercise(self):
@@ -246,14 +266,7 @@ class ExerciseTracker:
                 if exercises in [None,]:
                     break
                 precedence=None
-                def gethpv(routine):
-                    with Session(ENGINE) as session:
-                        hpv=session.query(Routine).filter(Routine.Name==routine.get("Name")).order_by(Routine.precedence.desc()).first()
-                        #print(hpv)
-                        if hpv:
-                            return hpv.precedence
-                        else:
-                            return 0
+
                 precedence=gethpv(routine)
                 for num,i in enumerate(exercises):
                     print(i)
@@ -274,6 +287,129 @@ class ExerciseTracker:
                     else:
                         pass
 
+
+    def Add2Routine(self):
+        with Session(ENGINE) as session:
+            fields={
+                'are you adding a new excercise or using an old one [New=True,False=Old]?':{
+                'default':True,
+                'type':'boolean',
+                },
+                'Routine ID(RID)':{
+                'default':None,
+                'type':'integer'
+                }
+            }
+            fd=FormBuilder(data=fields)
+            if fd is not None:
+                routine=session.query(Routine).filter(Routine.roid==fd['Routine ID(RID)']).first()
+                if routine is None:
+                    print(f"{Fore.orange_red_1}There is no Routine by that ID (Make sure it is an RID)")
+                    return
+
+                mode=fd['are you adding a new excercise or using an old one [New=True,False=Old]?']
+                if mode in [None,True]:
+                    excludes=['exid',]
+                    tmp=Exercise()
+                    session.add(tmp)
+                    session.commit()
+                    session.refresh(tmp)
+                    fields={
+                    i.name:{
+                        'default':getattr(tmp,i.name),
+                        'type':str(i.type).lower()
+                        } for i in tmp.__table__.columns if i.name not in excludes
+                    }
+                    fd=FormBuilder(data=fields,passThruText=f"Create a New Exercise\n To Attach to Routine(roid={routine.roid},RoutineName={routine.Name})")
+                    if fd is None:
+                        print(f"{Fore.orange_red_1}User Cancelled!{Style.reset}")
+                        session.delete(tmp)
+                        session.commit()
+                        return
+                    try:
+                        for k in fd:
+                            setattr(tmp,k,fd[k])
+                        session.commit()
+                        session.refresh(tmp)
+                        dtoe=datetime.now()
+                        doe=date(dtoe.year,dtoe.month,dtoe.day)
+                        toe=time(dtoe.hour,dtoe.minute,dtoe.second)
+                        nt=gethpv(routine)
+                        #print(nt)
+                        nt+=1
+                        rts=Routine(Name=getattr(routine,"Name"),Note=getattr(routine,"Note"),exid=tmp.exid,precedence=nt,doe=doe,dtoe=dtoe,toe=toe)
+                        session.add(rts)
+                        session.commit()
+                    except Exception as e:
+                        print(e,repr(e),str(e))
+                        session.rollback()
+                        session.commit()
+                elif mode in [False,]:
+                    excludes=['exid',]
+                    fields={
+                    'What are looking for':{
+                        'type':'string',
+                        'default':''
+                        }
+                    }
+                    fd=FormBuilder(data=fields,passThruText="What are looking to copy to this routine?")
+                    if fd is None:
+                        print(f"{Fore.orange_red_1}User Cancelled!{Style.reset}")
+                        return
+                    search=fd['What are looking for']
+                    OR=or_(Exercise.Name.icontains(search),Exercise.Note.icontains(search))
+                    AND=and_(Exercise.Name.icontains(search),Exercise.Note.icontains(search))
+
+                    query=session.query(Exercise).filter(or_(OR,AND))
+                    ordered=orderQuery(query,Exercise.Name)
+                    results=ordered.all()
+                    cta=len(results)
+                    htext=[]
+                    for num,i in enumerate(results):
+                        htext.append(std_colorize(i,num,cta))
+                    htext='\n'.join(htext)
+                    print(htext)
+                    whichIndexes=Control(func=FormBuilderMkText,ptext="Which index(es)?",helpText=f"{htext}\nseperate the indexes by commas",data="list")
+                    if whichIndexes in ['NaN',None,'d']:
+                        print(f"{Fore.orange_red_1}User Cancelled!{Style.reset}")
+                        return
+                    elif whichIndexes in [[],]:
+                        print(f"{Fore.orange_red_1}the list provided was empty, so user may have cancelled!{Style.reset}")
+                        return
+                    for index in whichIndexes:
+                        try:
+                            index=int(index)
+                            exercise=results[index]
+                            nex=Exercise()
+                            fields={
+                                i.name:{
+                                    'default':getattr(exercise,i.name),
+                                    'type':str(i.type).lower()
+                                    } for i in exercise.__table__.columns if i.name not in excludes
+                                }
+                            for f in fields:
+                                setattr(nex,f,getattr(exercise,f))
+                            session.add(nex)
+                            session.commit()
+                            session.refresh(nex)
+                            print(nex)
+                            dtoe=datetime.now()
+                            doe=date(dtoe.year,dtoe.month,dtoe.day)
+                            toe=time(dtoe.hour,dtoe.minute,dtoe.second)
+                            nt=gethpv(routine)
+                            #print(nt)
+                            nt+=1
+                            rts=Routine(Name=getattr(routine,"Name"),Note=getattr(routine,"Note"),exid=nex.exid,precedence=nt,doe=doe,dtoe=dtoe,toe=toe)
+                            session.add(rts)
+                            session.commit()
+                        except Exception as e:
+                            print(e)
+                            print(f"{Fore.magenta}Processing will attempt to continue!")
+                print("Added Exercise to Routine!")
+            else:
+                print(f"{Fore.orange_red_1}User Cancelled!{Style.reset}")
+                return
+
     def summarize(self):
         try:
             start=datetime.now()
@@ -363,6 +499,31 @@ class ExerciseTracker:
                         lite.append(r.Name)
                 if returnable:
                     return lite
+
+    def viewRoutineList(self,returnable=False):
+        with Session(ENGINE) as session:
+            query=session.query(Routine).order_by(Routine.roid)
+            results=query.all()
+            ct=len(results)
+            if ct < 1:
+                print(f"{Fore.light_red}No Routine's available!{Style.reset}")
+                return
+            else:
+                if returnable:
+                    lite=[]
+                for num,r in enumerate(results):
+                    ex=session.query(Exercise).filter(Exercise.exid==r.exid).first()
+                    if ex:
+                        name,note,exid=ex.Name,ex.Note,ex.exid
+                    else:
+                        name,note,exid="NotFound?","NotFound?","NotFound?"
+                    msg=f"{Fore.light_yellow}{num}/{Fore.light_red}{ct}{Style.reset}{Fore.light_steel_blue} -> {r.Name} Exercise(Name='{name}',Note='{note}',exid='{exid}'){Style.reset}"
+                    print(msg)
+                    if returnable:
+                        lite.append(r.Name)
+                if returnable:
+                    return lite
+
     def useRoutine(self,reset_only=False,reset_note=False):
         try:
             logMe=Prompt.__init2__(None,func=FormBuilderMkText,ptext="Log a comment in HealthLog(hl[default=True/Yes])?",helpText="yes or no/boolean",data="boolean")
@@ -431,7 +592,7 @@ class ExerciseTracker:
                             #return
                         total_dur=timedelta(seconds=0)
                         l=0
-                        while completed < ct:
+                        while completed < total_reps:
                             lapStart=datetime.now()
                             for num, i in enumerate(rout):
                                 if completed_reps > total_reps:
@@ -557,6 +718,8 @@ class ExerciseTracker:
         ee,EditExercise - edit an exercise
         nr,NewRoutine - a new routine of exercises
         vr,ViewRoutines - show available routines
+        vrl,ViewRoutines - show available routines and their exercises as a list
+        a2r,add2routine - add an exercise to a routine specified by RID
         ur,UseRoutine - use a set routine
         rr,RemoveRoutine - delete a routine
         cr,ClearRoutine - reset routine CurrentRep
@@ -581,6 +744,8 @@ class ExerciseTracker:
                 self.newRoutine()
             elif doWhat.lower() in ['vr','ViewRoutine'.lower()]:
                 self.viewRoutine()
+            elif doWhat.lower() in ['vlr','ViewRoutineList'.lower()]:
+                self.viewRoutineList()
             elif doWhat.lower() in ['ur','UseRoutine'.lower()]:
                 self.useRoutine()
             elif doWhat.lower() in ['rr','RemoveRoutine'.lower()]:
@@ -590,4 +755,6 @@ class ExerciseTracker:
             elif doWhat.lower() in ['hl','healthlog'.lower()]:
                 HealthLogUi()
             elif doWhat.lower() in "su,summarize".split(","):
-                self.summarize()
\ No newline at end of file
+                self.summarize()
+            elif doWhat.lower() in 'a2r,add2routine'.split(","):
+                self.Add2Routine()
\ No newline at end of file
